/*
 *
 *
 *    Resea (x86/kernel.h)
 *    Mafice <mafice@mflabo.net>
 *
 *
 */


//
// 型定義
//

typedef unsigned char  bool;
#define  true      1
#define  false     0

#define  NULL  ((void *) 0)

typedef signed char                s8;
typedef signed short int           s16;
typedef signed int                 s32;

typedef unsigned char              u8;
typedef unsigned short int         u16;
typedef unsigned int               u32;

typedef u32     count_t;
typedef u32     index_t;
typedef u32     address_t;
typedef u32     size_t;

typedef u16     segReg;
typedef u32     flagsReg;




// カーネルイメージの情報
#define  KERNELIMG_SIZE          0x10000
#define  INTERPRETER_SIZE        0x50000

// 各データのアドレス
#define  KERNELIMG_ADDRESS         0x8000
#define  BOOT4_ADDRESS             0x8c00
#define  INTERPRETER_IMG_ADDRESS   0x17e00
#define  VRAM_ADDRESS              0xb8000
#define  IDT_ADDRESS               0x100000
#define  TEMP_ADDRESS              0x120000
#define  GDT_ADDRESS               0x130000
#define  TSS_ADDRESS               0x140000
#define  IDTR_ADDRESS              TEMP_ADDRESS
#define  GDTR_ADDRESS              TEMP_ADDRESS
#define  MEMORY_USERAREA_ADDRESS   0x4000000
#define  INTERPRETER_ADDRESS       0x2000000
#define  INTERPRETER_STACK_ADDRESS 0x3000000


//インタプリタ
#define INTEPRETER_EFLAGS          0x202


// GDT
#define  GDT_LENGTH                      (16*7-1)
#define  GDT_KERNEL_CODE_SELECTOR        8*1
#define  GDT_KERNEL_DATA_SELECTOR        8*2
#define  GDT_USER_CODE_SELECTOR          8*3
#define  GDT_USER_DATA_SELECTOR          8*4
#define  GDT_TSS_SELECTOR                8*5
#define  GDT_LIMIT2                      0xc0
#define  GDT_KERNEL_CODESEG              0x9a
#define  GDT_KERNEL_DATASEG              0x92
#define  GDT_USER_CODESEG                0xfa
#define  GDT_USER_DATASEG                0xf2
#define  GDT_TSS                         0x89
#define  GDT_TSS_LIMIT                   0x2068

// IDT
#define  IDT_LENGTH               0x7ff
#define  IDT_HW_INT_SELECTOR      0x20
#define  IDT_API_SELECTOR         0x70
#define  IDT_INT_HANDLER          0x8e00
#define  IDT_API                  0xee00

// 表示設定
#define  CUI_DEFAULT_TEXT_COLOR   0x07
#define  CUI_DEFAULT_BG_COLOR     0x00

// メモリ
#define  MEMORY_BLOCK_SIZE          0x1000
#define  MEMORY_ENTRY_MAX           0x1000

// 割り込み
#define INTERRUPT_MASK_MASTER     0
#define INTERRUPT_MASK_SLAVE      0
#define ISR_MAX                   15

// IO ポート
#define IOPORT_INTMASK_MASTER     0x21
#define IOPORT_INTMASK_SLAVE      0xa1

// API
#define API_putchar       901
#define API_out           111
#define API_in            121
#define API_setISR        201
#define API_waitInterrupt 202


/* FIXME */
#include <stdarg.h>

// タスク

// TSS
#define  TSS_ESP0       0x8c00

// Screen
#define  SCREEN_X_MAX    80
#define  SCREEN_Y_MAX    25



//
//  構造体
//

struct st_TSS{
 u32      backlink;
 u32      esp0;
 u32      ss0;
 u32      esp1;
 u32      ss1;
 u32      esp2;
 u32      ss2;
 u32      cr3;
 u32      eip;
 u32      eflags;
 u32      eax;
 u32      ecx;
 u32      edx;
 u32      ebx;
 u32      esp;
 u32      ebp;
 u32      esi;
 u32      edi;
 segReg   es;
 segReg   cs;
 segReg   ss;
 segReg   ds;
 segReg   fs;
 segReg   gs;
 u32      ldt;
 u32      ioMapBase;
 u8       ioPermMap[8192];
} __attribute__((packed));


struct TSSDescriptor{
 u16   limit1;
 u16   base1;
 u8    base2;
 u8    info;
 u8    limit2;
 u8    base3;
}__attribute__ ((packed));


struct SegDescriptor{
 u16   limit1;
 u16   base1;
 u8    base2;
 u8    info;
 u8    limit2;
 u8    base3;
}__attribute__ ((packed));


struct IntDescriptor{
 u16      offset1;
 segReg   cs;
 u16      info;
 u16      offset2;
}__attribute__ ((packed));


struct st_GDTR{
  u16   length;
  u32   address;
}__attribute__ ((packed));


struct st_IDTR{
  u16   length;
  u32   address;
}__attribute__ ((packed));


struct InterruptInfo{
  address_t isr[ISR_MAX];
};


struct ScreenInfo{
  unsigned int  x;
  unsigned int  y;
  unsigned int  xMax;
  unsigned int  yMax;
  unsigned int  textColor;
  unsigned int  bgColor;
  address_t     VRAMAddress;
};


struct MemoryEntry{

  enum {
    MEMORY_UNUSED,
    MEMORY_USING
  } status;

};


struct MemoryInfo{
  address_t             capacity;
  struct MemoryEntry    block[MEMORY_ENTRY_MAX];
};




//
// グローバル変数
//

extern struct MemoryInfo     Memory;
extern struct ScreenInfo     Screen;
extern struct InterruptInfo  Interrupt;




//
//  関数
//

// timer.c
void initTimer (void);


// halt.c
void halt (void);


//memory,c
bool isActiveMemoryAddress (address_t addr);
void *kmalloc              (size_t memsize);


// descriptor.c
void setSegDescriptor (index_t dsc, address_t base, address_t limit,  u16 info);
void setIntDescriptor (index_t dsc, segReg    cs,   address_t offset, u16 info);
void setTSSDescriptor (index_t dsc, address_t base, u16       info);



// string.c
int           strncmp (const char *str1 , const char *str2 , size_t len);
size_t        strlen  (const char *str);
int           strcmp  (const char *str1, const char *str2);
char         *strcpy  (char       *str1, const char *str2);
void          memcpy  (void       *to,   void       *from, size_t size);



// halt.c
void   halt  (void);



// assembly.asm
void asm_startupInterpreter (address_t offset, segReg cs, segReg ds, address_t stack, flagsReg eflags);

void   asm_hlt     (void);
void   asm_cli     (void);
void   asm_sti     (void);
void   asm_ltr     (u16 ltr);
void   asm_lgdt    (u32 gdtr);
void   asm_lidt    (u32 idtr);
void   asm_out8    (u16 port, u8 data);
u8     asm_in8     (u16 port);
void   asm_setCR3  (u32 cr3);
u32    asm_loadCR2 (void);



// print.c
void   print       (const char *format, ...);
void   printchar   (const char ch, const unsigned char color);
int    vprintf     (const char *format, va_list vargs);



// intHandler.asm
void  asm_APIEntry            (void);
void  asm_intTimerHandler     (void);

void asm_hwIntHandler0  (void);
void asm_hwIntHandler1  (void);
void asm_hwIntHandler2  (void);
void asm_hwIntHandler3  (void);
void asm_hwIntHandler4  (void);
void asm_hwIntHandler5  (void);
void asm_hwIntHandler6  (void);
void asm_hwIntHandler7  (void);
void asm_hwIntHandler8  (void);
void asm_hwIntHandler9  (void);
void asm_hwIntHandler10 (void);
void asm_hwIntHandler11 (void);
void asm_hwIntHandler12 (void);
void asm_hwIntHandler13 (void);
void asm_hwIntHandler14 (void);
void asm_hwIntHandler15 (void);




// exception.c
void  asm_exception0         (void);
void  asm_exception1         (void);
void  asm_exception2         (void);
void  asm_exception3         (void);
void  asm_exception4         (void);
void  asm_exception5         (void);
void  asm_exception6         (void);
void  asm_exception7         (void);
void  asm_exception8         (void);
void  asm_exception9         (void);
void  asm_exception10        (void);
void  asm_exception11        (void);
void  asm_exception12        (void);
void  asm_exception13        (void);
void  asm_exception14        (void);
void  asm_exception15        (void);
void  asm_exception16        (void);
void  asm_exception17        (void);
void  asm_exception18        (void);
void  asm_exception19        (void);

